
#include "comb/mixedradix-gray.h"

#include "aux1/copy.h" // fill()

#include "demo/nextarg.h" // NXARG()
#include "fxttypes.h"
#include "fxtalloca.h"
#include "fxtio.h"
#include "fxtiomanip.h"
#include "jjassert.h"


//% Mixed radix Gray code, CAT algorithm.

//#define TIMING // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 3;
    NXARG(n,"Number of digits");

    ulong rr = 4;
    NXARG(rr, "Base (radix) of digits (0==>falling factorial, 1==>rising factorial)");

    ALLOCA(ulong, r, n);
    fill(r, n, rr);
    RESTARGS("Optionally supply radix for all digits (rr ignored)");
    if ( argc>3)  jjassert( argc == (int)n+3 );
    for (ulong k=3;  k<(ulong)argc; ++k)  r[k-3] = atol(argv[k]);

    mixedradix_gray M(n, rr, (argc>3? r:0) );
    M.print_nines("Nines: ");
    cout << endl;

//    M.last();
    ulong ct = 0;
    do
    {
#ifndef TIMING
        cout << " " << setw(4) << ct << ":  ";
        M.print("    ", 1 );
        cout << setw(6) << M.to_num();
        cout << setw(6) << M.pos() << "  " << (M.dir()<0?'-':'+') << 1;

//        cout << "    ";
//        for (ulong j=0; j<n; ++j)  cout << " " << ((long)(M.i_[j])<0?'-':'+');

        cout << endl;
#endif // TIMING
        ++ct;
    }
    while ( M.next() );
//    while ( M.prev() );

    cout << " ct=" << ct << endl;

    return 0;
}
// -------------------------

/*
Timing:

 time ./bin 30 2  ## binary is worst case
 ct=1073741824
./bin 30 2  9.01s user 0.05s system 100% cpu 9.056 total
 ==> 1073741824/9.01 == 119,172,233 per second

 time ./bin 16 4
 ct=4294967296
./bin 16 4  21.25s user 0.11s system 99% cpu 21.368 total
 ==> 4294967296/21.25 == 202,116,108 per second

 time ./bin 10 8
 ct=1073741824
./bin 10 8  4.36s user 0.02s system 99% cpu 4.388 total
 ==> 1073741824/4.36 == 246,271,060 per second

 time ./bin 8 16
 ct=4294967296
./bin 8 16  18.50s user 0.09s system 99% cpu 18.591 total
 ==> 4294967296/18.50 == 232,160,394 per second

*/



/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/mixedradix-gray-demo.cc"
/// End:

