
#include "comb/mixedradix-gray2.h"

#include "comb/comb-print.h"

#include "aux1/copy.h" // fill()

#include "demo/nextarg.h" // NXARG()
#include "fxttypes.h"
#include "fxtalloca.h"
#include "fxtio.h"
#include "fxtiomanip.h"
#include "jjassert.h"


//% Mixed radix Gray code, loopless algorithm.

//#define TIMING // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 3;
    NXARG(n,"Number of digits");

    ulong rr = 4;
    NXARG(rr, "Base (radix) of digits (0==>falling factorial, 1==>rising factorial)");

    ALLOCA(ulong, r, n);
    fill(r, n, rr);
    RESTARGS("Optionally supply radix for all digits (rr ignored)");
    if ( argc>3)  jjassert( argc == (int)n+3 );
    for (ulong k=3;  k<(ulong)argc; ++k)  r[k-3] = atol(argv[k]);

    mixedradix_gray2 M(n, rr, (argc>3? r:0) );
    M.print_nines("Nines: ");
    cout << endl;

//    M.last();
    ulong ct = 0;
    do
    {
#ifndef TIMING
        cout << " " << setw(4) << ct << ":  ";
        M.print("    ", 1 );
        cout << setw(6) << M.to_num();
        cout << setw(6) << M.pos() << "  " << (M.dir()<0?'-':'+') << 1;
        print_vec("    f=", M.f_, n+1);  // focus pointer incl. sentinel
        cout << endl;
#endif // TIMING
        ++ct;
    }
    while ( M.next() );
//    while ( M.prev() );

    cout << " ct=" << ct << endl;

    return 0;
}
// -------------------------

/*
Timing:

 time ./bin 30 2  ## binary is worst case
 ct=1073741824
./bin 30 2  5.73s user 0.02s system 99% cpu 5.756 total
 ==> 1073741824/5.73 == 187,389,498 per second

 time ./bin 16 4
 ct=4294967296
./bin 16 4  19.85s user 0.12s system 99% cpu 19.972 total
 ==> 4294967296/19.85 == 216,371,148 per second

 time ./bin 10 8
 ct=1073741824
./bin 10 8  4.78s user 0.02s system 99% cpu 4.808 total
 ==> 1073741824/4.78 == 224,632,180 per second

 time ./bin 8 16
 ct=4294967296
./bin 8 16  17.16s user 0.09s system 99% cpu 17.253 total
 ==> 4294967296/17.16 == 250,289,469 per second

*/



/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/mixedradix-gray2-demo.cc"
/// End:

