
#include "comb/mixedradix-lex.h"

#include "aux1/copy.h" // fill()

#include "demo/nextarg.h" // NXARG()
#include "fxttypes.h"
#include "fxtalloca.h"
#include "fxtio.h"
#include "jjassert.h"

//% Mixed radix counting.


//#define TIMING // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 3;
    NXARG(n,"Number of digits");

    ulong rr = 4;
    NXARG(rr, "Base (radix) of digits (0==>falling factorial, 1==>rising factorial)");

    ALLOCA(ulong, r, n);
    fill(r, n, rr);
    RESTARGS("Optionally supply radix for all digits (rr ignored)");
    if ( argc>3)  jjassert( argc == (int)n+3 );
    for (ulong k=3;  k<(ulong)argc; ++k)  r[k-3] = atol(argv[k]);

    mixedradix_lex M(n, rr, (argc>3? r:0) );
    M.print_nines("Nines: ");
    cout << endl;

//    M.last();
    ulong ct = 0;
    do
    {
#ifndef TIMING
        cout << " " << setw(4) << ct << ":  ";
        M.print("    ", 1 );
        cout << setw(6) << M.to_num();  // == ct
        cout << endl;
#endif // TIMING
        ++ct;
    }
    while ( M.next() );
//    while ( M.prev() );

    cout << " ct=" << ct << endl;

    return 0;
}
// -------------------------

/*
Timing:
 time ./bin 30 2  ## binary is worst case
 ct=1073741824
./bin 30 2  7.42s user 0.04s system 99% cpu 7.470 total
 ==> 1073741824/7.42 == 144,709,140 per second

 time ./bin 19 3
 ct=1162261467
./bin 19 3  5.44s user 0.02s system 99% cpu 5.469 total
 ==> 1162261467/5.44 == 213,651,004 per second

 time ./bin 16 4
ct=4294967296
./bin 16 4  16.71s user 0.07s system 99% cpu 16.785 total
 ==> 4294967296/16.71 == 257,029,760 per second

 time ./bin 10 8
 ct=1073741824
./bin 10 8  3.30s user 0.02s system 99% cpu 3.329 total
 ==> 1073741824/3.30 == 325,376,310 per second

 time ./bin 8 16
 ct=4294967296
./bin 8 16  14.74s user 0.07s system 99% cpu 14.810 total
 ==> 4294967296/14.74 == 291,381,770 per second


 time ./bin 12 1  ## rising factorial
Nines: [ 1 2 3 4 5 6 7 8 9 10 11 12 ]
 ct=6227020800
./bin 12 1  25.11s user 0.13s system 99% cpu 25.235 total
 ==> 6227020800/25.11 == 247,989,677 per second

 time ./bin 12 0  ## falling factorial
Nines: [ 12 11 10 9 8 7 6 5 4 3 2 1 ]
 ct=6227020800
./bin 12 0  22.46s user 0.11s system 99% cpu 22.577 total
 ==> 6227020800/22.46 == 277,249,367 per second

*/

/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/mixedradix-lex-demo.cc"
/// End:

