
#include "comb/mixedradix-modular-gray.h"

#include "aux1/copy.h" // fill()

#include "demo/nextarg.h" // NXARG()
#include "fxttypes.h"
#include "fxtalloca.h"
#include "fxtio.h"
#include "fxtiomanip.h"
#include "jjassert.h"

//% Modular mixed radix Gray code.

//#define TIMING // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 3;
    NXARG(n,"Number of digits");

    ulong rr = 4;
    NXARG(rr, "Base (radix) of digits (0==>falling factorial, 1==>rising factorial)");

    ALLOCA(ulong, r, n);
    fill(r, n, rr);
    RESTARGS("Optionally supply radix for all digits (rr ignored)");
    if ( argc>3)  jjassert( argc == (int)n+3 );
    for (ulong k=3;  k<(ulong)argc; ++k)  r[k-3] = atol(argv[k]);

    mixedradix_modular_gray M(n, rr, (argc>3? r:0) );
    M.print_nines("Nines: ");
    cout << endl;

//    M.last();
    ulong ct = 0;
    do
    {
#ifndef TIMING
        cout << " " << setw(4) << ct << ":  ";
        M.print("    ", 1 );
//        cout << setw(6) << M.to_num();
        cout << setw(6) << M.pos();
        cout << endl;
#endif // TIMING
        ++ct;
    }
    while ( M.next() );
//    while ( M.prev() );

    cout << " ct=" << ct << endl;

    return 0;
}
// -------------------------

/*
Timing:

 time ./bin 30 2  ## binary is worst case
 ct=1073741824
./bin 30 2  7.86s user 0.03s system 99% cpu 7.898 total
 ==> 1073741824/7.86 == 136,608,374 per second

 time ./bin 16 4
 ct=4294967296
./bin 16 4  22.86s user 0.12s system 99% cpu 22.989 total
 ==> 4294967296/22.86 == 187,881,334 per second

 time ./bin 10 8
 ct=1073741824
./bin 10 8  4.64s user 0.02s system 99% cpu 4.670 total
 ==> 1073741824/4.64 == 231,409,875 per second

 time ./bin 8 16
 ct=4294967296
./bin 8 16  18.86s user 0.10s system 99% cpu 18.959 total
 ==> 4294967296/18.86 == 227,728,912 per second
*/

/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/mixedradix-modular-gray-demo.cc"
/// End:

