#if !defined HAVE_BITBUTTERFLY_H__
#define      HAVE_BITBUTTERFLY_H__

#include "fxttypes.h"
#include "bits/bitsperlong.h"


//: The butterfly_*() functions are used in bit_zip() and bit_unzip()


#if  BITS_PER_LONG == 64
static inline ulong butterfly_16(ulong x)
{
    const ulong ml = 0x0000ffff00000000UL;
    const ulong s = 16;
    const ulong mr = ml >> s;
    const ulong t = ((x & ml) >> s ) | ((x & mr) << s );
    x = (x & ~(ml | mr)) | t;
    return  x;
}
// -------------------------
#endif



static inline ulong butterfly_8(ulong x)
{
#if  BITS_PER_LONG == 64
    const ulong ml = 0x00ff000000ff0000UL;
#else
    const ulong ml = 0x00ff0000UL;
#endif
    const ulong s = 8;
    const ulong mr = ml >> s;
    const ulong t = ((x & ml) >> s ) | ((x & mr) << s );
    x = (x & ~(ml | mr)) | t;
    return  x;
}
// -------------------------


static inline ulong butterfly_4(ulong x)
{
#if  BITS_PER_LONG == 64
    const ulong ml = 0x0f000f000f000f00UL;
#else
    const ulong ml = 0x0f000f00UL;
#endif
    const ulong s = 4;
    const ulong mr = ml >> s;
    const ulong t = ((x & ml) >> s ) | ((x & mr) << s );
    x = (x & ~(ml | mr)) | t;
    return  x;
}
// -------------------------


static inline ulong butterfly_2(ulong x)
{
#if  BITS_PER_LONG == 64
    const ulong ml = 0x3030303030303030UL;
#else
    const ulong ml = 0x30303030UL;
#endif
    const ulong s = 2;
    const ulong mr = ml >> s;
    const ulong t = ((x & ml) >> s ) | ((x & mr) << s );
    x = (x & ~(ml | mr)) | t;
    return  x;
}
// -------------------------


static inline ulong butterfly_1(ulong x)
{
#if  BITS_PER_LONG == 64
    const ulong ml = 0x4444444444444444UL;
#else
    const ulong ml = 0x44444444UL;
#endif
    const ulong s = 1;
    const ulong mr = ml >> s;
    const ulong t = ((x & ml) >> s ) | ((x & mr) << s );
    x = (x & ~(ml | mr)) | t;
    return  x;
}
// -------------------------


// Equivalent functions to butterfly_*()
// can be found at Torsten Sillkes bitmani.h, cf.
// http://www.mathematik.uni-bielefeld.de/~sillke/ALGORITHMS/


#endif  // !defined HAVE_BITBUTTERFLY_H__
